@@ -1,12 +1,12 @@
-# (c) Alexander Solovyov, 2009-2010, under terms of the new BSD License
+# (c) Alexander Solovyov, 2009-2011, under terms of the new BSD License
 '''Command line arguments parser
 '''
 
-import sys, traceback, getopt, types, textwrap, inspect, os
+import sys, traceback, getopt, types, textwrap, inspect, os, copy
 from itertools import imap
 
 __all__ = ['command', 'dispatch']
-__version__ = '1.1'
+__version__ = '2.2'
 __author__ = 'Alexander Solovyov'
 __email__ = 'piranha@piranha.org.ua'
 
@@ -18,12 +18,15 @@ try:
 except locale.Error:
     ENCODING = 'UTF-8'
 
-def write(text, out=sys.stdout):
+def write(text, out=None):
+    '''Write output to a given stream (stdout by default)'''
+    out = out or sys.stdout
     if isinstance(text, unicode):
         return out.write(text.encode(ENCODING))
     out.write(text)
 
 def err(text):
+    '''Write output to stderr'''
     write(text, out=sys.stderr)
 
 CMDTABLE = {}
@@ -37,7 +40,7 @@ def command(options=None, usage=None, name=None, shortlist=False, hide=False):
 
     All arguments are optional:
 
-     - ``options``: options in format described in docs. If not supplied,
+     - ``options``: options in format described later. If not supplied,
        will be determined from function.
      - ``usage``: usage string for function, replaces ``%name`` with name
        of program or subcommand. In case if it's subcommand and ``%name``
@@ -48,6 +51,21 @@ def command(options=None, usage=None, name=None, shortlist=False, hide=False):
        only with multiple subcommands
      - ``hide``: if command should be hidden from help listing. Used only
        with multiple subcommands, overrides ``shortlist``
+
+    Options should be a list of 4-tuples in format::
+
+      (shortname, longname, default, help)
+
+    Where:
+
+     - ``shortname`` is a single letter which can be used then as an option
+       specifier on command line (like ``-a``). Will be not used if contains
+       falsy value (empty string, for example)
+     - ``longname`` - main identificator of an option, can be used as on a
+       command line with double dashes (like ``--longname``)
+     - ``default`` value for an option, type of it determines how option will be
+       processed
+     - ``help`` string displayed as a help for an option when asked to
     '''
     def wrapper(func):
         try:
@@ -69,6 +87,7 @@ def command(options=None, usage=None, name=None, shortlist=False, hide=False):
 
         def help_func(name=None):
             return help_cmd(func, replace_name(usage_, sysname()), options_)
+        func.help = help_func
 
         @wraps(func)
         def inner(*args, **opts):
@@ -81,23 +100,23 @@ def command(options=None, usage=None, name=None, shortlist=False, hide=False):
 
             argv = opts.pop('argv', sys.argv[1:])
             if opts.pop('help', False):
-                return help_func()
+                return func.help()
 
             if args or opts:
                 # no catcher here because this is call from Python
                 return call_cmd_regular(func, options_)(*args, **opts)
 
             try:
-                opts, args = catcher(lambda: parse(argv, options_), help_func)
+                opts, args = catcher(lambda: parse(argv, options_), func.help)
             except Abort:
                 return -1
 
             if opts.pop('help', False):
-                return help_func()
+                return func.help()
 
             try:
                 return catcher(lambda: call_cmd(name_, func)(*args, **opts),
-                               help_func)
+                               func.help)
             except Abort:
                 return -1
 
@@ -149,8 +168,9 @@ def dispatch(args=None, cmdtable=None, globaloptions=None,
     if name == '_completion':       # skip middleware
         worker = lambda: call_cmd(name, func)(*args, **kwargs)
     else:
-        worker = lambda: (call_cmd(name, middleware(func), depth=2)
-                          (*args, **kwargs))
+        mwfunc = middleware(func)
+        depth = func == mwfunc and 1 or 2
+        worker = lambda: call_cmd(name, mwfunc, depth=depth)(*args, **kwargs)
 
     try:
         return catcher(worker, help_func)
@@ -162,6 +182,8 @@ def dispatch(args=None, cmdtable=None, globaloptions=None,
 # --------
 
 def help_(cmdtable, globalopts):
+    '''Help generator for a command table
+    '''
     def help_inner(name=None):
         '''Show help for a given help topic or a help overview
 
@@ -181,8 +203,8 @@ def help_(cmdtable, globalopts):
                 if shortlist and not cmd.startswith('^'):
                     continue  # short help contains only marked commands
                 cmd = cmd.lstrip('^~')
-                doc = info[0].__doc__ or '(no help text available)'
-                hlp[cmd] = doc.splitlines()[0].rstrip()
+                doc = pretty_doc_string(info[0])
+                hlp[cmd] = doc.strip().splitlines()[0].rstrip()
 
             hlplist = sorted(hlp)
             maxlen = max(map(len, hlplist))
@@ -191,11 +213,7 @@ def help_(cmdtable, globalopts):
             write('\ncommands:\n\n')
             for cmd in hlplist:
                 doc = hlp[cmd]
-                if False:  # verbose?
-                    write(' %s:\n     %s\n' % (cmd.replace('|', ', '), doc))
-                else:
-                    write(' %-*s  %s\n' % (maxlen, cmd.split('|', 1)[0],
-                                              doc))
+                write(' %-*s  %s\n' % (maxlen, cmd.split('|', 1)[0], doc))
 
         if not cmdtable:
             return err('No commands specified!\n')
@@ -242,15 +260,16 @@ def help_cmd(func, usage, options):
      -p --port       port to listen on (default: 8000)
      -d --daemonize  daemonize process
         --pid-file   name of file to write process ID to
-    <BLANKLINE>
     '''
     write(usage + '\n')
-    doc = func.__doc__ or '(no help text available)'
+    doc = pretty_doc_string(func)
     write('\n' + doc.strip() + '\n\n')
     if options:
         write(''.join(help_options(options)))
 
 def help_options(options):
+    '''Generator for help on options
+    '''
     yield 'options:\n\n'
     output = []
     for o in options:
@@ -309,8 +328,8 @@ def parse(args, options):
         defmap[pyname] = default
 
         # copy defaults to state
-        if isinstance(default, list):
-            state[pyname] = default[:]
+        if isinstance(default, (list, dict)):
+            state[pyname] = copy.copy(default)
         elif hasattr(default, '__call__'):
             funlist.append(pyname)
             state[pyname] = None
@@ -319,8 +338,10 @@ def parse(args, options):
 
         # getopt wants indication that it takes a parameter
         if not (default is None or default is True or default is False):
-            if short: short += ':'
-            if name: name += '='
+            if short:
+                short += ':'
+            if name:
+                name += '='
         if short:
             shortlist += short
         if name:
@@ -335,8 +356,15 @@ def parse(args, options):
         if t is types.FunctionType:
             del funlist[funlist.index(name)]
             state[name] = defmap[name](val)
-        elif t is types.ListType:
+        elif t is list:
             state[name].append(val)
+        elif t is dict:
+            try:
+                k, v = val.split('=')
+            except ValueError:
+                raise ParseError(name, "wrong definition: '%s' "
+                                 "(should be in format KEY=VALUE)" % val)
+            state[name][k] = v
         elif t in (types.NoneType, types.BooleanType):
             state[name] = not defmap[name]
         else:
@@ -353,6 +381,8 @@ def parse(args, options):
 # --------
 
 def _dispatch(args, cmdtable, globalopts):
+    '''Dispatch arguments list by a command table
+    '''
     cmd, func, args, options = cmdparse(args, cmdtable, globalopts)
 
     if options.pop('help', False):
@@ -363,6 +393,8 @@ def _dispatch(args, cmdtable, globalopts):
     return cmd, func, args, options
 
 def cmdparse(args, cmdtable, globalopts):
+    '''Parse arguments list to find a command, options and arguments
+    '''
     # command is the first non-option
     cmd = None
     for arg in args:
@@ -381,14 +413,11 @@ def cmdparse(args, cmdtable, globalopts):
 
     possibleopts.extend(globalopts)
 
-    try:
-        options, args = parse(args, possibleopts)
-    except getopt.GetoptError, e:
-        raise ParseError(cmd, e)
-
-    return (cmd, cmd and info[0] or None, args, options)
+    options, args = parse(args, possibleopts)
+    return cmd, cmd and info[0] or None, args, options
 
 def aliases_(cmdtable_key):
+    '''Get aliases from a command table key'''
     return cmdtable_key.lstrip("^~").split("|")
 
 def findpossible(cmd, table):
@@ -434,7 +463,16 @@ def findcmd(cmd, table):
 # --------
 
 def guess_options(func):
-    args, varargs, varkw, defaults = inspect.getargspec(func)
+    '''Get options definitions from function
+
+    They should be declared in a following way:
+
+    def func(longname=(shortname, default, help)):
+        pass
+
+    See docstring of ``command()`` for description of those variables.
+    '''
+    args, _, _, defaults = inspect.getargspec(func)
     for name, option in zip(args[-len(defaults):], defaults):
         try:
             sname, default, hlp = option[:3]
@@ -444,6 +482,8 @@ def guess_options(func):
             pass
 
 def guess_usage(func, options):
+    '''Get usage definition for a function
+    '''
     usage = '%name '
     if options:
         usage += '[OPTIONS] '
@@ -470,7 +510,7 @@ def catcher(target, help_func):
             (e.args[0], ' '.join(e.args[1])))
         raise Abort()
     except ParseError, e:
-        err('%s: %s\n' % (e.args[0], e.args[1]))
+        err('%s: %s\n\n' % (e.args[0], e.args[1].strip()))
         help_func(e.args[0])
         raise Abort()
     except getopt.GetoptError, e:
@@ -496,6 +536,8 @@ def call_cmd(name, func, depth=1):
     return inner
 
 def call_cmd_regular(func, opts):
+    '''Wrapper for command for handling function calls from Python
+    '''
     def inner(*args, **kwargs):
         funcargs, _, varkw, defaults = inspect.getargspec(func)
         if len(args) > len(funcargs):
@@ -511,11 +553,13 @@ def call_cmd_regular(func, opts):
     return inner
 
 def replace_name(usage, name):
+    '''Replace name placeholder with a command name'''
     if '%name' in usage:
         return usage.replace('%name', name, 1)
     return name + ' ' + usage
 
 def sysname():
+    '''Returns name of executing file'''
     name = sys.argv[0]
     if name.startswith('/'):
         return name.rsplit('/', 1)[1]
@@ -523,11 +567,22 @@ def sysname():
         return name[2:]
     return name
 
+def pretty_doc_string(item):
+    "Doc string with adjusted indentation level of the 2nd line and beyond."
+    raw_doc = item.__doc__ or '(no help text available)'
+    lines = raw_doc.strip().splitlines()
+    if len(lines) <= 1:
+        return raw_doc
+    indent = len(lines[1]) - len(lines[1].lstrip())
+    return '\n'.join([lines[0]] + map(lambda l: l[indent:], lines[1:]))
+
 try:
     from functools import wraps
 except ImportError:
     def wraps(wrapped, assigned=('__module__', '__name__', '__doc__'),
               updated=('__dict__',)):
+        '''functools.wraps replacement for Python 2.4
+        '''
         def inner(wrapper):
             for attr in assigned:
                 setattr(wrapper, attr, getattr(wrapped, attr))
@@ -644,3 +699,7 @@ class ParseError(OpsterError):
 
 class Abort(OpsterError):
     'Processing error, abort execution'
+
+if __name__ == '__main__':
+    import doctest
+    doctest.testmod()
